//********************************************************************************
//
// Perlin Noise class.
//
// Original author: Rob Baker.
// Current maintainer: Rob Baker.
//
//********************************************************************************

#pragma once
#pragma managed(push,off)


//********************************************************************************
// Include the required header files.
//********************************************************************************

#include "randomnumbers.h"


namespace libmaths
{

	//********************************************************************************
	// Perlin Noise class.
	//********************************************************************************

	class PerlinNoise
	{

	public:

		// Construct the Perlin object with a seed for the noise.
		PerlinNoise(void)
		{ Initialise(0xdeadbeef); }

		// Initialise the Perlin object with a seed for the noise.
		void Initialise(U32 seed);

		// Get some one dimensional noise.
		F32 Noise1(F32 x);

		// Get some two dimensional noise.
		F32 Noise2(F32 x, F32 y);

		// Get some three dimensional noise.
		F32 Noise3(F32 x, F32 y, F32 z);

	private:

		static const S32 SIZE = 256;
		static const S32 MASK = SIZE - 1;

		// Permutation table
		U8 _p[SIZE];

		// Gradients
		F32 _gx[SIZE];
		F32 _gy[SIZE];
		F32 _gz[SIZE];

		RandomFast _random;
	};

// namespace libmaths
}

#pragma managed(pop)
