//********************************************************************************
//
// Spherical coordinates representing a 3D vector.
// Consists of a radius from the origin, a inclination angle from the +Y axis and
// an azimuth angle from the +X axis.
//
// http://en.wikipedia.org/wiki/Spherical_coordinates
//
// Original author: Rob Baker.
// Current maintainer: Rob Baker.
//
//********************************************************************************

#pragma once
#pragma managed(push,off)


//********************************************************************************
// Include the required header files.
//********************************************************************************

#include "spherical3.h"


namespace libmaths
{

	//********************************************************************************
	// Spherical coordinates representing a 3D vector.
	// Consists of a radius from the origin, a inclination angle from the +Z axis and
	// an azimuth angle from the +X axis.
	//********************************************************************************

	class Spherical3LH : public Spherical3
	{

	public:

		// Get the epsilon value for equality comparisons.
		static const F32 Epsilon(void)
		{ return 1.0e-5f; }

		// Default constructor.
		Spherical3LH(void)
		{ }

		// Construct from azimuth and inclination.
		Spherical3LH(F32 azimuth, F32 inclination) :
			Spherical3(azimuth, inclination)
		{ }

		// Construct from azimuth, inclination and radius.
		Spherical3LH(F32 azimuth, F32 inclination, F32 radius) :
			Spherical3(azimuth, inclination, radius)
		{ }

		// Construct from a Vector3x, in left-hand coordinate system.
		Spherical3LH(const Vector3x& rhs)
		{ *this = rhs; }

		// Assignment from a Vector3x, in left-hand coordinate system.
		Spherical3LH& operator = (const Vector3x& rhs);

		// Assignment from a Vector3, in left-hand coordinate system.
		Spherical3LH& operator = (const Vector3& rhs);

		// Compare two coordinates for equality.
		Bool operator == (const Spherical3LH& rhs) const
		{ return ((Vector3x(*this) - Vector3x(rhs)).LengthSqr() < Epsilon() * Epsilon()); }

		// Compare two coordinates for inequality.
		Bool operator != (const Spherical3LH& rhs) const
		{ return !((*this) == rhs); }
	};

	// Now include the inline definitions.

#include "spherical3lh.inl"

// namespace libmaths
}

#pragma managed(pop)
