//********************************************************************************
//
// 3D vector class.
// Most commonly used for 3D work - in 3 and 4 component flavours (4th component
// is ignored but is present for performance reasons).
//
// Original author: Rob Baker.
// Current maintainer: Rob Baker.
//
//********************************************************************************

#pragma once
#pragma managed(push,off)


//********************************************************************************
// Include the Vector3 class member functions.
//********************************************************************************

#define VECTOR3_TYPE Vector3
#define OTHER_VECTOR3_TYPE Vector3x


//********************************************************************************
// Construct from other vector type.
//********************************************************************************

LIBMATHS_INLINE VECTOR3_TYPE::VECTOR3_TYPE(const OTHER_VECTOR3_TYPE& p)
: x(p.x), y(p.y), z(p.z)
{
}


//********************************************************************************
// Assignment from another vector.
//********************************************************************************

LIBMATHS_INLINE VECTOR3_TYPE& VECTOR3_TYPE::operator = (const VECTOR3_TYPE& rhs)
{

#if LIBMATHS_32BIT

	*(S32*)&x = *(S32*)&rhs.x;
	*(S32*)&y = *(S32*)&rhs.y;
	*(S32*)&z = *(S32*)&rhs.z;

#elif LIBMATHS_64BIT

	*(S64*)&x = *(S64*)&rhs.x;
	*(S32*)&z = *(S32*)&rhs.z;

#else

	x = rhs.x;
	y = rhs.y;
	z = rhs.z;

#endif

	return *this;
}


//********************************************************************************
// Assignment from other vector type.
//********************************************************************************

LIBMATHS_INLINE VECTOR3_TYPE& VECTOR3_TYPE::operator = (const OTHER_VECTOR3_TYPE& rhs)
{

#if LIBMATHS_32BIT

	*(S32*)&x = *(S32*)&rhs.x;
	*(S32*)&y = *(S32*)&rhs.y;
	*(S32*)&z = *(S32*)&rhs.z;

#elif LIBMATHS_64BIT

	*(S64*)&x = *(S64*)&rhs.x;
	*(S32*)&z = *(S32*)&rhs.z;

#else

	x = rhs.x;
	y = rhs.y;
	z = rhs.z;

#endif

	return *this;
}

#include "vector3common.inl"

#undef OTHER_VECTOR3_TYPE
#undef VECTOR3_TYPE


//********************************************************************************
// Include the Vector3x class member functions.
//********************************************************************************

#define VECTOR3_TYPE Vector3x
#define OTHER_VECTOR3_TYPE Vector3


//********************************************************************************
// Construct from other vector type.
//********************************************************************************

LIBMATHS_INLINE VECTOR3_TYPE::VECTOR3_TYPE(const OTHER_VECTOR3_TYPE& p)
: x(p.X()), y(p.Y()), z(p.Z()), w(0)
{
}


//********************************************************************************
// Assignment from another vector.
//********************************************************************************

LIBMATHS_INLINE VECTOR3_TYPE& VECTOR3_TYPE::operator = (const VECTOR3_TYPE& rhs)
{

#if LIBMATHS_32BIT

	*(S32*)&x = *(S32*)&rhs.x;
	*(S32*)&y = *(S32*)&rhs.y;
	*(S32*)&z = *(S32*)&rhs.z;
	*(S32*)&w = *(S32*)&rhs.w;

#elif LIBMATHS_64BIT

	*(S64*)&x = *(S64*)&rhs.x;
	*(S64*)&z = *(S64*)&rhs.z;

#else

	x = rhs.x;
	y = rhs.y;
	z = rhs.z;
	w = rhs.w;

#endif

	return *this;
}


//********************************************************************************
// Assignment from other vector type.
//********************************************************************************

LIBMATHS_INLINE VECTOR3_TYPE& VECTOR3_TYPE::operator = (const OTHER_VECTOR3_TYPE& rhs)
{

#if LIBMATHS_32BIT

	*(S32*)&x = *(S32*)&rhs.x;
	*(S32*)&y = *(S32*)&rhs.y;
	*(S32*)&z = *(S32*)&rhs.z;
	*(S32*)&w = 0;

#elif LIBMATHS_64BIT

	*(S64*)&x = *(S64*)&rhs.x;
	*(S32*)&z = *(S32*)&rhs.z;
	*(S32*)&w = 0;

#else

	x = rhs.x;
	y = rhs.y;
	z = rhs.z;
	w = 0.0f;

#endif

	return *this;
}

#include "vector3common.inl"

#undef OTHER_VECTOR3_TYPE
#undef VECTOR3_TYPE

#pragma managed(pop)
