//********************************************************************************
//
// 2D circles.
//
// Circles can detect overlaps with other circles, points and line segments.
//
// Original author: Rob Baker.
// Current maintainer: Rob Baker.
//
//********************************************************************************


//********************************************************************************
// Include the required header files.
//********************************************************************************

#include "circle.h"
#include "rectangle.h"


namespace libmaths
{

	//********************************************************************************
	// Construct circle to enclose the given rectangle.
	//********************************************************************************

	Circle::Circle(const AxisAlignedRectangle& rectangle)
	{
		_centre = rectangle.Centre();
		_radius = (rectangle.Maximum() - rectangle.Minimum()).Length() * 0.5f;
	}


	//********************************************************************************
	// Build circle enclosing the two given spheres.
	//********************************************************************************

	void Circle::Surround(const Circle& circle0, const Circle& circle1)
	{
		AxisAlignedRectangle rectangle, s1b(circle0), s2b(circle1);

		rectangle.Surround(s1b, s2b);
		_centre = rectangle.Centre();
		Vector2 ext(rectangle.Maximum() - rectangle.Minimum());
		_radius = ext.Length() * 0.5f;
	}


	//********************************************************************************
	// Build circle enclosing the given circle and rectangle.
	//********************************************************************************

	void Circle::Surround(const Circle& circle, const AxisAlignedRectangle& rectangle)
	{
		AxisAlignedRectangle b, b2(circle);

		b.Surround(rectangle, b2);
		_centre = b.Centre();
		_radius = (b.Maximum() - b.Minimum()).Length() * 0.5f;
	}


	//********************************************************************************
	//  Extend the circle to additionally contain the given circle.
	//********************************************************************************

	void Circle::Add(const Circle& circle)
	{
		Surround(*this, circle);
	}


	//********************************************************************************
	// Extend the circle to additionally contain the given rectangle.
	//********************************************************************************

	void Circle::Add(const AxisAlignedRectangle& rectangle)
	{
		Surround(*this, rectangle);
	}


	//********************************************************************************
	// Check if the ray intersects this circle.
	// Does not allow intersections from within the circle.
	//********************************************************************************

	Bool Circle::Intersects(const Ray2& ray) const
	{
		// TODO: Write this code.
		LIBMATHS_ASSERT(0 && "Not yet implemented");

		return false;
	}


	//********************************************************************************
	// Check if the ray intersects this circle.
	// Does not allow intersections from within the circle.
	//********************************************************************************

	Bool Circle::Intersects(const Ray2& ray, Vector2& point, Vector2& normal) const
	{
		// TODO: Write this code.
		LIBMATHS_ASSERT(0 && "Not yet implemented");

		return false;
	}

// namespace libmaths
}
