//********************************************************************************
//
// Frustum - 5 to 6 sided viewing volume normally associated with a camera.
//
// Original author: Rob Baker.
// Current maintainer: Rob Baker.
//
//********************************************************************************


//********************************************************************************
// Include the required header files.
//********************************************************************************

#include "frustum.h"


namespace libmaths
{
	//********************************************************************************
	// Setup a frustum from a field of view and near / far distances.
	//********************************************************************************

	void Frustum::FromFOVDistances(F32 verticalFOV, F32 aspectRatio, F32 nearDistance, F32 farDistance)
	{
		F32 horizontalFOV = verticalFOV * aspectRatio;

		for (S32 i = Frustum::FRUSTUM_LEFT; i <= Frustum::FRUSTUM_BOTTOM; i++)
		{
			Position(i, Vector3x::ZERO());
		}

		Vector3x plane;

		plane = Vector3x(+libmaths::Cos(horizontalFOV * 0.5f), libmaths::Sin(horizontalFOV * 0.5f), 0.0f);
		plane.Normalise();
		Normal(Frustum::FRUSTUM_LEFT, plane);

		plane = Vector3x(-libmaths::Cos(horizontalFOV * 0.5f), libmaths::Sin(horizontalFOV * 0.5f), 0.0f);
		plane.Normalise();
		Normal(Frustum::FRUSTUM_RIGHT, plane);

		plane = Vector3x(0.0f, libmaths::Sin(verticalFOV * 0.5f), +libmaths::Cos(verticalFOV * 0.5f));
		plane.Normalise();
		Normal(Frustum::FRUSTUM_TOP, plane);

		plane = Vector3x(0.0f, libmaths::Sin(verticalFOV * 0.5f), -libmaths::Cos(verticalFOV * 0.5f));
		plane.Normalise();
		Normal(Frustum::FRUSTUM_BOTTOM, plane);

		Normal(Frustum::FRUSTUM_FRONT, Vector3x(0.0f, 1.0f, 0.0f));
		Position(Frustum::FRUSTUM_FRONT, Vector3x(0.0f, nearDistance, 0.0f));

		Normal(Frustum::FRUSTUM_BACK, Vector3x(0.0f, -1.0f, 0.0f));
		Position(Frustum::FRUSTUM_BACK, Vector3x(0.0f, farDistance, 0.0f));
	}

// namespace libmaths
}
