//********************************************************************************
//
// Core maths implementation.
//
// Original author: Rob Baker.
// Current maintainer: Rob Baker.
//
//********************************************************************************


//********************************************************************************
// Include the required header files.
//********************************************************************************

#include "libmaths.h"


//********************************************************************************
// Calculate the roots of the quadratic equation ax^2 + bx + c = 0.
// The number of valid roots is returned by the function, and the roots themselves
// are placed into the two dimensional array of floats.
//********************************************************************************

S32 libmaths::QuadraticRoots(F32 a, F32 b, F32 c, F32* pRoots)
{
	F32 d, q;
	S32 count = 0;

	d = (b * b) - (4.0f * a * c);

	if (d < 0.0f)
	{
		*pRoots = *(pRoots + 1) = 0.0f;

		return 0;
	}

	d = Sqrt(d);

	q = ((b < 0.0f) ? -(0.5f * ((b - d))) : -(0.5f * ((b + d))));

	if (a != 0.0f)
	{
		*pRoots = q / a;

		pRoots++;
		count++;
	}

	if (q != 0.0f)
	{
		*pRoots = c / q;

		count++;
	}

	return count;
}
