//********************************************************************************
//
// This class allows planes to be defined and tested for collision with rays
// across the game world.
//
// Original author: Rob Baker.
// Current maintainer: Rob Baker.
//
//********************************************************************************


//********************************************************************************
// Include the required header files.
//********************************************************************************

#include "plane.h"


namespace libmaths
{

	//********************************************************************************
	// Check if the ray intersects this plane.
	//********************************************************************************

	Bool Plane::Intersects(const Ray3& ray, Bool twoSided) const
	{
		F32 cosAlpha = ray.Direction() * Normal();

		if (cosAlpha >= 0.0f)
		{
			if (twoSided == false || cosAlpha == 0.0f)
			{
				return false;
			}
		}

		F32 Delta = (Normal() * (Origin() - ray.Origin())) / cosAlpha;

		if (Delta < 0.0f)
		{
			return false;
		}

		return true;
	}


	//********************************************************************************
	// Check if the ray intersects this plane.
	//********************************************************************************

	Bool Plane::Intersects(const Ray3& ray, Vector3x& point, Vector3x& normal, Bool twoSided) const
	{
		F32 cosAlpha = ray.Direction() * Normal();

		if (cosAlpha >= 0.0f)
		{
			if (twoSided == false || cosAlpha == 0.0f)
			{
				return false;
			}
		}

		F32 Delta = (Normal() * (Origin() - ray.Origin())) / cosAlpha;

		if (Delta < 0.0f)
		{
			return false;
		}

		point = ray.Origin() + (ray.Direction() * Delta);
		normal = Normal();

		return true;
	}

// namespace libmaths
}
