//********************************************************************************
//
// A class used to define rays in 2D space, often used for detecting intersections
// with other geometric types such as circles and triangles.
//
// Original author: Rob Baker.
// Current maintainer: Rob Baker.
//
//********************************************************************************


//********************************************************************************
// Include the required header files.
//********************************************************************************

#include "ray2.h"
#include "rectangle.h"
#include "circle.h"
#include "triangle2.h"


namespace libmaths
{

	//********************************************************************************
	// Does the ray intersect a rectangle?
	//********************************************************************************

	Bool Ray2::Intersects(AxisAlignedRectangle& rectangle) const
	{
		return rectangle.Intersects(*this);
	}


	//********************************************************************************
	// Does the ray intersect a rectangle?
	//********************************************************************************

	Bool Ray2::Intersects(AxisAlignedRectangle& rectangle, Vector2& point, Vector2& normal, F32& t) const
	{
		return rectangle.Intersects(*this, point, normal);
	}


	//********************************************************************************
	// Does the ray intersect a circle?
	//********************************************************************************

	Bool Ray2::Intersects(Circle& circle) const
	{
		return circle.Intersects(*this);
	}


	//********************************************************************************
	// Does the ray intersect a circle?
	//********************************************************************************

	Bool Ray2::Intersects(Circle& circle, Vector2& point, Vector2& normal, F32& t) const
	{
		return circle.Intersects(*this, point, normal);
	}


	//********************************************************************************
	// Does the ray intersect a triangle?
	//********************************************************************************

	Bool Ray2::Intersects(Triangle2& triangle) const
	{
		return triangle.Intersects(*this);
	}


	//********************************************************************************
	// Does the ray intersect a triangle?
	//********************************************************************************

	Bool Ray2::Intersects(Triangle2& triangle, Vector2& point, Vector2& normal, F32& t) const
	{
		return triangle.Intersects(*this, point, normal);
	}

// namespace libmaths
}